package net.xdocc;
import java.io.File;
import java.util.List;

import net.xdocc.virtualfilesystem.VirtualDirectory;
import net.xdocc.virtualfilesystem.VirtualFile;
import net.xdocc.virtualfilesystem.VirtualNode;

import org.apache.commons.io.FileUtils;

public class ContentCopy implements ContentCompiler
{
	final private Site site;
	final private VirtualFile virtualFile;

	public ContentCopy(Site site, VirtualFile virtualFile)
	{
		this.site = site;
		this.virtualFile = virtualFile;
	}

	@Override
	public Boolean call() throws Exception
	{
		if (!virtualFile.getBackingFile().exists())
		{
			System.err.println("the file " + virtualFile + " does not exist!!");
			return false;
		}
		File genDir = new File(site.getGenerated(), virtualFile.getRelativeVisiblePath());
		genDir.mkdirs();
		File dst = new File(genDir, virtualFile.getAttributes().getVisibleName());
		if (!site.getCache().isValid(virtualFile.getBackingFile(),dst))
		{
			
			FileUtils.copyFile(virtualFile.getBackingFile(), dst);
			System.err.println("copy " + virtualFile.getBackingFile() + " to " + dst);
			site.getCache().addGenerated(virtualFile.getBackingFile(), dst);
		}
		return true;
	}

	@Override
	public Entry view(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current) throws Exception
	{
		if (!virtualFile.getBackingFile().exists())
		{
			System.err.println("the file " + virtualFile + " does not exist!!");
			return null;
		}
		if (virtualNode.getAttributes().isVisible())
		{
			String url = Utils.createRelativePathWeb(virtualNode, current)
					+ virtualNode.getAttributes().getVisibleName();
			return new Entry("Download: <a href=\"" + url + "\">"
					+ virtualNode.getAttributes().getVisibleName() + "</a>, Size: "
					+ virtualNode.getBackingFile().length() + " bytes", 1, virtualFile.getBackingFile());
		}
		else
			return new Entry("", 1, virtualFile.getBackingFile());
	}

	@Override
	public List<Document> viewCompiled(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current)
	{
		return null;
	}
}
