package net.xdocc;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

import net.xdocc.virtualfilesystem.VirtualDirectory;
import net.xdocc.virtualfilesystem.VirtualFile;
import net.xdocc.virtualfilesystem.VirtualNode;

public class DirectoryCompiler implements Callable<Boolean>
{
	final private VirtualDirectory virtualDirectory;
	final private Site site;
	final private VirtualDirectory current;

	public DirectoryCompiler(Site site, VirtualDirectory virtualDirectory, VirtualDirectory current)
	{
		this.virtualDirectory = virtualDirectory;
		this.site = site;
		this.current = current;
	}

	@Override
	public Boolean call() throws Exception
	{
		File generated = Utils.createGenerated(site, virtualDirectory, "index.html");
		if (!site.getCache().isValid(virtualDirectory.getBackingFile(), generated))
		{
			try
			{
				List<Document> documents = view();
				if (documents.size() > 0)
				{
					Utils.writeHTML(site, virtualDirectory, virtualDirectory, documents
							.toArray(new Document[] {}), generated);
				}
				return true;
			}
			catch (Exception e)
			{
				e.printStackTrace();
				return false;
			}
		}
		return true;
	}

	public List<Document> view() throws Exception
	{
		return view(virtualDirectory, false, false);
	}

	public List<Document> view(VirtualDirectory linkTo, boolean forceNotseamless, boolean preview)
			throws Exception
	{
		List<Document> tmp = new ArrayList<Document>();
		return viewRec(site, virtualDirectory, tmp, 0, linkTo, current, forceNotseamless, preview);
	}

	private List<Document> viewRec(Site site, VirtualDirectory virtualDirectory,
			List<Document> tmp, int depth, VirtualNode linkTo, VirtualDirectory current,
			boolean forceNotseamless, boolean preview) throws Exception
	{
		if (virtualDirectory.getVisibleFiles().size() > 0)
		{
			preview = preview || depth != 0;
			List<VirtualFile> virtualFile2 = new ArrayList<VirtualFile>();
			for (VirtualFile virtualFile : virtualDirectory.getVisibleFiles())
			{
				if (virtualFile.getAttributes().getTags().contains("highlight"))
				{
					virtualFile2.add(virtualFile);
				}
			}
			if (virtualDirectory.getAttributes().getTags().contains("all") || !preview)
			{
				// here we dont need any link, since we show everything
				Document doc = Utils.createDocument(site, virtualDirectory, linkTo, current, depth);
				for (VirtualFile virtualFile : virtualDirectory.getVisibleFiles())
				{
					ContentCompiler cc = Contents.getFileHandler(site, virtualFile);
					Entry e = cc.view(site, virtualFile, linkTo, current);
					if (e == null)
					{
						List<Document> tt = cc.viewCompiled(site, virtualFile, linkTo, current);
						if(tt!=null)
							tmp.addAll(tt);
					}
					else if(!virtualFile.getAttributes().isDate())
						doc.addContent(e);
					else
					{
						Document doc2 = Utils.createDocument(site, virtualFile, virtualFile, current, depth);
						doc2.addContent(e);
						if (doc2.hasEntry())
							tmp.add(doc2);
					}
					
				}
				if (doc.hasEntry())
					tmp.add(doc);
			}
			else
			{
				List<VirtualFile> tmp2=new ArrayList<VirtualFile>(virtualDirectory.getVisibleFiles());
				for(VirtualFile diff:virtualFile2)
					tmp.remove(diff);
				virtualFile2.addAll(tmp2);
				// if we have only one file, do not show link, since we show
				// everything.
				Document doc = Utils.createDocument(site, virtualDirectory, linkTo, current, depth);
				int i = 0;
				for (VirtualFile virtualFile : virtualFile2)
				{
					ContentCompiler cc = Contents.getFileHandler(site, virtualFile);
					Entry e = cc.view(site, virtualFile, linkTo, current);
					if (e == null)
					{
						List<Document> tt = cc.viewCompiled(site, virtualFile, linkTo, current);
						if(tt!=null)
							tmp.addAll(tt);
					}
					else
					{
						doc.addContent(e);
						i++;
						if (e.getModulo() == i)
							break;
					}
				}
				if (doc.hasEntry())
					tmp.add(doc);
			}
		}
		else if (virtualDirectory.getVisibleDirectories().size() > 0)
		{
			if (depth == 0)
			{
				for (VirtualDirectory virtualDirectory2 : virtualDirectory.getVisibleDirectories())
				{
					viewRec(site, virtualDirectory2, tmp, depth + 1, virtualDirectory2, current,
							forceNotseamless, preview);
				}
			}
			else
			{
				VirtualDirectory virtualDirectory2 = virtualDirectory.getVisibleDirectories()
						.get(0);
				for (VirtualDirectory virtualDirectory3 : virtualDirectory.getVisibleDirectories())
				{
					if (virtualDirectory3.getAttributes().getTags().contains("highlight"))
					{
						virtualDirectory2 = virtualDirectory3;
						break;
					}
				}
				viewRec(site, virtualDirectory2, tmp, depth + 1, linkTo, current, forceNotseamless,
						preview);
			}
		}
		return tmp;
	}
}
