package net.xdocc;
import java.io.File;
import java.io.IOException;
import java.util.List;

import net.xdocc.virtualfilesystem.VirtualDirectory;
import net.xdocc.virtualfilesystem.VirtualFile;
import net.xdocc.virtualfilesystem.VirtualNode;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

public class ImageHandler implements ContentCompiler
{
	final private Site site;
	final private VirtualNode virtualNode;

	public ImageHandler(Site site, VirtualNode virtualNode)
	{
		this.site = site;
		this.virtualNode = virtualNode;
	}

	@Override
	public Boolean call() throws Exception
	{
		if (!virtualNode.getBackingFile().exists())
		{
			System.err.println("the file " + virtualNode + " does not exist!!");
			return false;
		}
		if (!site.getCache().isValid(virtualNode.getBackingFile()))
		{
			File picture = virtualNode.getBackingFile();
			VirtualFile virtualFile = (VirtualFile) virtualNode;
			if (virtualNode.getParent().getAttributes().isVisible())
				generate(site, virtualFile);
			else
			{
				File dst1 = new File(site.getGenerated(), virtualFile.getRelativeVisiblePath());
				File dst = new File(dst1, picture.getName());
				FileUtils.copyFile(picture, dst);
				site.getCache().addGenerated(virtualFile.getBackingFile(), dst);
			}
		}
		else
			System.err.println("file "+virtualNode.getBackingFile()+" is cached");
		return true;
	}

	@Override
	public Entry view(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current) throws Exception
	{
		File picture = virtualNode.getBackingFile();
		String baseName = "img" + VirtualNode.getIndex((VirtualFile) virtualNode);
		String thumbName = baseName + "t." + FilenameUtils.getExtension(picture.getName());
		String origName = baseName + "." + FilenameUtils.getExtension(picture.getName());
		String thumbURL = Utils.createRelativePathWeb(virtualNode, current) + thumbName;
		String origURL = Utils.createRelativePathWeb(virtualNode, current) + origName;
		return new Entry("<div id=\"image\"><a href=\"" + origURL + "\"><img src=\"" + thumbURL
				+ "\" border=\"0\"/></a><div id=\"caption\">"
				+ FilenameUtils.getBaseName(virtualNode.getAttributes().getVisibleName())
				+ "</div></div>", 3, picture);
	}

	@Override
	public List<Document> viewCompiled(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current)
	{
		return null;
	}

	static String[] generate(Site site, VirtualFile virtualFile) throws IOException
	{
		System.err.println("process Image" + virtualFile);
		String baseName = "img" + VirtualNode.getIndex(virtualFile);
		File picture = virtualFile.getBackingFile();
		String ext = FilenameUtils.getExtension(picture.getName());
		String path=virtualFile.getRelativeVisiblePath();
		if(path.length()>0)
			path=path+"/";
		String thumbName = path + baseName + "t." + ext;
		String normName = path + baseName + "n." + ext;
		String origName = path + baseName + "." + ext;
		if (site.getCache().generatedCount(virtualFile.getBackingFile()) == 4
				&& site.getCache().isValid(virtualFile.getBackingFile()))
			return new String[] { thumbName, normName, origName };
		// if (test.contains(virtualFile.getBackingFile()))
		// throw new RuntimeException("nope!! " + virtualFile);
		// test.add(virtualFile.getBackingFile());
		File thumb = new File(site.getGenerated(), thumbName);
		File norm = new File(site.getGenerated(), normName);
		File orig = new File(site.getGenerated(), origName);
		thumb.getParentFile().mkdirs();
		norm.getParentFile().mkdirs();
		int width = site.getImageWidth();
		int height = site.getImageHeight();
		int width_thumb = site.getImageThumbWidth();
		int height_thumb = site.getImageThumbHeight();
		Utils.resize(picture, thumb, width_thumb, height_thumb);
		site.getCache().addGenerated(virtualFile.getBackingFile(), thumb);
		Utils.resize(picture, norm, width, height);
		site.getCache().addGenerated(virtualFile.getBackingFile(), norm);
		FileUtils.copyFile(picture, orig);
		site.getCache().addGenerated(virtualFile.getBackingFile(), orig);
		return new String[] { thumbName, normName, origName };
	}
}
