package net.xdocc;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

public class PropertyReader
{
	final private static String KEY = "key_tag_header";
	final private Map<String, List<Properties>> tags = new LinkedHashMap<String, List<Properties>>();
	final private Properties globalProperties = new Properties();

	public PropertyReader(File property) throws IOException
	{
		FileInputStream fstream = new FileInputStream(property);
		BufferedReader br = new BufferedReader(new InputStreamReader(fstream));
		try
		{
			String line = null;
			boolean tagMode = false;
			Properties properties = null;
			while ((line = br.readLine()) != null)
			{
				line = line.trim();
				if (!(line.startsWith("#") || line.equals("")))
				{
					int startTag = line.indexOf("<");
					int endTag = line.lastIndexOf(">");
					if (startTag == 0 && endTag > 0)
					{
						int endTagName = line.indexOf("/");
						int startKey = line.indexOf("=");
						if (endTagName > startTag && endTagName < endTag && startKey < 0)
						{
							tagMode = false;
							properties = null;
						}
						else if (startKey > startTag && startKey < endTag)
						{
							tagMode = true;
							String key = line.substring(startTag + 1, startKey);
							String value = line.substring(startKey + 1, endTag);
							properties = new Properties();
							properties.setProperty(KEY, value);
							addTags(key, properties);
						}
						else
							throw new IOException("syntax error in property file in line [" + line
									+ "]");
					}
					else
					{
						Properties p = tagMode ? properties : globalProperties;
						int split = line.indexOf("=");
						if (split > 0)
						{
							String key = line.substring(0, split);
							if (key.equals(KEY))
								throw new IOException("Key " + key + " is reserved");
							String value = line.substring(split + 1);
							p.setProperty(key, value);
						}
						else
							p.setProperty(line, "true");
					}
				}
			}
		}
		finally
		{
			br.close();
			fstream.close();
		}
	}

	private void addTags(String key, Properties properties)
	{
		List<Properties> propertyList = tags.get(key);
		if (propertyList == null)
		{
			propertyList = new ArrayList<Properties>();
			tags.put(key, propertyList);
		}
		propertyList.add(properties);
	}

	public String get(String key)
	{
		return globalProperties.getProperty(key);
	}

	public List<Properties> getTag(String key)
	{
		return tags.get(key);
	}

	public static String getKey(Properties properties)
	{
		return properties.getProperty(KEY);
	}
}
