package net.xdocc;
import java.io.File;
import java.io.IOException;

import net.xdocc.virtualfilesystem.VirtualFS;

import org.apache.velocity.Template;
import org.apache.velocity.app.VelocityEngine;

public class Site implements Comparable<Site>
{
	public final static String SETTINGS_NAME = "settings.conf";
	public final static String TEMPLATE_NAME = "template.html.vm";
	final private VirtualFS contentVirtualFS;
	final private VirtualFS templateVirtualFS;
	final private File generated;
	final private VelocityEngine ve;
	final private File settings;
	final private Cache cache;
	private PropertyReader propertyReader;
	private Template template;
	private boolean changed;

	public Site(VirtualFS contentVirtualFS, VirtualFS templateVirtualFS, File generated,
			VelocityEngine ve, File settings, Cache cache) throws Exception
	{
		this.contentVirtualFS = contentVirtualFS;
		this.templateVirtualFS = templateVirtualFS;
		this.generated = generated;
		this.ve = ve;
		this.settings = settings;
		this.cache = cache;
		readSettings();
		readTemplate();
	}

	public VirtualFS getContentVirtualFS()
	{
		return contentVirtualFS;
	}

	public VirtualFS getTemplateVirtualFS()
	{
		return templateVirtualFS;
	}

	public File getGenerated()
	{
		return generated;
	}

	public Template getTemplate()
	{
		return template;
	}

	public PropertyReader getProperties()
	{
		return propertyReader;
	}

	public Cache getCache()
	{
		return cache;
	}

	public void readSettings() throws IOException
	{
		this.propertyReader = new PropertyReader(settings);
	}

	public void readTemplate() throws Exception
	{
		this.template = ve.getTemplate(TEMPLATE_NAME);
	}

	/*
	 * public String getWebRoot() { return "/JumpseatDMS/" + name + "/"; }
	 */
	public int getNavigation1Depth()
	{
		return 1;
	}

	public int getNavigation2Depth()
	{
		return 2;
	}

	public int getNavigation1From()
	{
		return 1;
	}

	public int getNavigation2From()
	{
		return 10;
	}

	public String getDateFormat()
	{
		return "yyyy-mm-dd";
	}

	public int getImageWidth()
	{
		return 800;
	}

	public int getImageHeight()
	{
		return 800;
	}

	public int getImageThumbWidth()
	{
		return 300;
	}

	public int getImageThumbHeight()
	{
		return 300;
	}

	@Override
	public int compareTo(Site o)
	{
		return generated.compareTo(o.generated);
	}

	@Override
	public boolean equals(Object obj)
	{
		if (obj instanceof Site)
			return compareTo((Site) obj) == 0;
		else
			return false;
	}

	@Override
	public int hashCode()
	{
		return generated.hashCode();
	}

	public boolean isChanged()
	{
		return changed;
	}

	public void setChanged(boolean changed)
	{
		this.changed = changed;
	}

	public void shutdown()
	{
		contentVirtualFS.dispose();
		templateVirtualFS.dispose();
		cache.shutdown();
	}
}
