package net.xdocc;
import java.io.File;
import java.util.List;

import net.xdocc.virtualfilesystem.VirtualDirectory;
import net.xdocc.virtualfilesystem.VirtualFile;
import net.xdocc.virtualfilesystem.VirtualNode;

import org.apache.commons.io.FileUtils;

public class TextHandler implements ContentCompiler
{
	final static int PREVIEW_CHARS = 500;
	final private Site site;
	final private VirtualFile virtualNode;

	public TextHandler(Site site, VirtualFile virtualNode)
	{
		this.site = site;
		this.virtualNode = virtualNode;
	}

	@Override
	public Boolean call() throws Exception
	{
		if (!virtualNode.getBackingFile().exists())
		{
			System.err.println("the file " + virtualNode + " does not exist!!");
			return false;
		}
		// do not compile files that do not have a date, these will
		// be compiled in index.dir.html
		if ((virtualNode.getParent().getVisibleFiles().size() <= 1 || !virtualNode.getParent()
				.hasFullEntries())
				&& (virtualNode.getParent().getAttributes().isVisible() || virtualNode.getParent()
						.isRoot()))
		{
			return false;
		}
		File generated = Utils.createGenerated(site, virtualNode, virtualNode.getAttributes()
				.getURLName()
				+ ".html");
		if (!site.getCache().isValid(virtualNode.getBackingFile(), generated))
		{
			Document doc = Utils.createDocument(site, virtualNode, virtualNode, virtualNode,0);
			Entry entry = view(site, virtualNode, null, virtualNode.getParent());
			doc.addContent(entry);
			Utils.writeHTML(site, virtualNode, virtualNode.getParent(), new Document[] { doc },
					generated);
		}
		return true;
	}

	@Override
	public Entry view(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current) throws Exception
	{
		if (!virtualNode.getBackingFile().exists())
		{
			System.err.println("the file " + virtualNode + " does not exist!!");
			return null;
		}
		String content = "<pre>" + FileUtils.readFileToString(virtualNode.getBackingFile())
				+ "</pre>";
		content = content.replace("\n", "<br/>");
		return new Entry("<div id=\"txt_full\">" + content + "</div>", 1, virtualNode
				.getBackingFile());
	}

	@Override
	public List<Document> viewCompiled(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current)
	{
		return null;
	}
}
