package net.xdocc;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.imageio.ImageIO;


import net.xdocc.virtualfilesystem.VirtualDirectory;
import net.xdocc.virtualfilesystem.VirtualFile;
import net.xdocc.virtualfilesystem.VirtualNode;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.velocity.VelocityContext;

public class Utils
{
	public static final char EXTENSION_SEPARATOR = '.';
	public static final char UNIX_SEPARATOR = '/';
	public static final char WINDOWS_SEPARATOR = '\\';

	public static String makeURL(String webRoot, VirtualDirectory vfile)
	{
		StringBuilder sb = new StringBuilder(webRoot);
		List<String> names = new ArrayList<String>();
		VirtualDirectory vdir = vfile;
		while (!vdir.isRoot())
		{
			names.add(0, vdir.getAttributes().getURLName());
			vdir = vdir.getParent();
		}
		Iterator<String> iterator = names.iterator();
		while (iterator.hasNext())
		{
			String name = iterator.next();
			sb.append(name);
			if (iterator.hasNext())
				sb.append("/");
		}
		sb.append("/");
		return sb.toString();
	}

	public static String makeURL(String[] tokens, int offset)
	{
		StringBuilder sb = new StringBuilder();
		for (int i = offset; i < tokens.length; i++)
			sb.append("/").append(tokens[i]);
		return sb.toString();
	}

	public static String[] getExtensions(String filename)
	{
		if (filename == null)
			return null;
		List<Integer> indexes = indexOfExtension(filename);
		if (indexes.size() == 0)
			return new String[0];
		else
		{
			Iterator<Integer> iterator = indexes.iterator();
			int start = iterator.next();
			if (!iterator.hasNext())
				return new String[] { filename.substring(start + 1) };
			String[] ret = new String[indexes.size()];
			for (int i = 0; iterator.hasNext(); i++)
			{
				int end = iterator.next();
				ret[i] = filename.substring(start + 1, end);
				start = end;
				if (!iterator.hasNext())
					ret[i + 1] = filename.substring(start + 1);
			}
			return ret;
		}
	}

	private static List<Integer> indexOfExtension(String filename)
	{
		List<Integer> tmp = new ArrayList<Integer>();
		if (filename == null)
			return tmp;
		int lastSeparator = indexOfLastSeparator(filename);
		int extensionPos = filename.indexOf(EXTENSION_SEPARATOR, lastSeparator + 1);
		int lastExtensionPos = -1;
		// dot files are anyway ignored
		while (extensionPos >= 0)
		{
			if (lastExtensionPos + 1 == extensionPos && lastExtensionPos >= 0)
				tmp.remove((Integer) lastExtensionPos);
			tmp.add(extensionPos);
			lastExtensionPos = extensionPos;
			extensionPos = filename.indexOf(EXTENSION_SEPARATOR, extensionPos + 1);
		}
		return tmp;
	}

	private static int indexOfLastSeparator(String filename)
	{
		if (filename == null)
			return -1;
		int lastUnixPos = filename.lastIndexOf(UNIX_SEPARATOR);
		int lastWindowsPos = filename.lastIndexOf(WINDOWS_SEPARATOR);
		return Math.max(lastUnixPos, lastWindowsPos);
	}

	public static String makeRelativePath(String fullPath, String absolutePath)
	{
		int start = fullPath.indexOf(absolutePath);
		if (start != 0)
			throw new RuntimeException("Not a relative path??");
		return fullPath.substring(absolutePath.length());
	}

	public static boolean resize(File picture, File thumb, int width, int height)
	{
		try
		{
			BufferedImage bufferedImage = ImageIO.read(picture);
			float scale1 = height / (float) bufferedImage.getHeight();
			float scale2 = width / (float) bufferedImage.getWidth();
			float scale = Math.min(scale1, scale2);
			int w = Math.round(bufferedImage.getWidth() * scale);
			int h = Math.round(bufferedImage.getHeight() * scale);
			BufferedImage resized = getScaledInstance(bufferedImage, w, h,
					RenderingHints.VALUE_INTERPOLATION_BICUBIC);
			ImageIO.write(resized, (bufferedImage.getTransparency() == Transparency.OPAQUE) ? "jpg"
					: "png", thumb);
		}
		catch (IOException e)
		{
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public static BufferedImage getScaledInstance(BufferedImage img, int targetWidth,
			int targetHeight, Object hint)
	{
		int type = (img.getTransparency() == Transparency.OPAQUE) ? BufferedImage.TYPE_INT_RGB
				: BufferedImage.TYPE_INT_ARGB;
		BufferedImage tmp = new BufferedImage(targetWidth, targetHeight, type);
		Graphics2D g2 = tmp.createGraphics();
		g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, hint);
		g2.drawImage(img, 0, 0, targetWidth, targetHeight, null);
		g2.dispose();
		return tmp;
	}

	public static int countLevels(VirtualNode root, VirtualNode current)
	{
		if (root.equals(current))
			return 0;
		int level = 0;
		VirtualDirectory tmp;
		if(current instanceof VirtualFile)
			tmp = current.getParent();
		else 
			tmp=(VirtualDirectory)current;
		while (tmp != null && !root.equals(tmp))
		{
			level++;
			tmp = tmp.getParent();
		}
		return level;
	}

	public static void writeHTML(Site site, VirtualNode virtualNode, VirtualDirectory current,
			Document[] documents, File generated) throws Exception
	{
		int paging = 5;
		int max = ((documents.length - 1) / paging) + 1;
		Link[] tmp = new Link[max];
		String base = FilenameUtils.getBaseName(generated.getName());
		String ext = FilenameUtils.getExtension(generated.getName());
		for (int i = 0; i < max; i++)
		{
			String name;
			if (i > 0)
				name = base + "_" + i + "." + ext;
			else
				name = generated.getName();
			//
			tmp[i] = new Link(name, Integer.toString(i + 1), null);
		}
		for (int i = 0; i < max; i++)
		{
			List<Document> tmp2 = new ArrayList<Document>();
			for (int j = 0; j < 5 && ((paging * i) + j) < documents.length; j++)
			{
				tmp2.add(documents[(paging * i) + j]);
			}
			File generated2 = Utils.createGenerated(site, virtualNode, tmp[i].getURL());
			writeHTML(site, virtualNode, current, tmp2.toArray(new Document[] {}), generated2, tmp);
		}
	}

	public static void writeHTML(Site site, VirtualNode virtualNode, VirtualDirectory current,
			Document[] documents, File generated, Link[] pages) throws Exception
	{
		VirtualFile indexTemplate = site.getTemplateVirtualFS().waitFor("template.html.vm");
		if (indexTemplate == null)
			throw new FileNotFoundException("template.html.vm not found");
		VirtualDirectory root = site.getContentVirtualFS().getRoot();
		if (root == null)
			throw new FileNotFoundException("contentQuickNavDir not found");
		//
		System.err.println("write " + generated);
		//
		String webRoot = Utils.createRelativePathWeb(site, current);
		VelocityContext context = new VelocityContext();
		context.put("path", webRoot);
		context.put("quicknav_links", doNav(site, webRoot, root, 2, generated));
		context.put("breandcrumb_links", doBreadcrumbs(site, webRoot, virtualNode, generated));
		if (current.getVisibleFiles().size() > 0)
			context.put("detailednav_links", doNav(site, webRoot, root, virtualNode, 2, 1,
					generated));
		else
			context.put("detailednav_links", new Link[] {});
		context.put("documents", documents);
		if(pages.length>1)
			context.put("page_links", pages);
		//
		FileWriter fw = new FileWriter(generated);
		StringWriter sw = new StringWriter();
		site.getTemplate().merge(context, sw);
		fw.write(sw.getBuffer().toString());
		fw.close();
		// add generated files
		// put this after the generated files, or the cacheitem will not
		// be generated
		site.getCache().addGenerated(virtualNode.getBackingFile(), generated);
		site.getCache().addDependency(virtualNode.getBackingFile(), indexTemplate.getBackingFile());
		site.getCache().addGenerated(indexTemplate.getBackingFile(), generated);
		for (Document doc : documents)
		{
			for (Entry entry : doc.getEntries())
				site.getCache().addDependency(virtualNode.getBackingFile(), entry.getSource());
		}
	}

	public static File createGenerated(Site site, VirtualNode virtualNode, String filename)
	{
		File generated;
		if (virtualNode instanceof VirtualDirectory)
		{
			VirtualDirectory vdir = (VirtualDirectory) virtualNode;
			generated = new File(site.getGenerated(), vdir.getRelativeVisiblePath());
			generated.mkdirs();
			generated = new File(generated, filename);
		}
		else
		{
			VirtualFile vfile = (VirtualFile) virtualNode;
			generated = new File(site.getGenerated(), vfile.getRelativeVisiblePath());
			generated.mkdirs();
			generated = new File(generated, filename);
		}
		return generated;
	}

	public static String createRelativePathWeb(Site site, VirtualDirectory current)
	{
		VirtualDirectory root = site.getContentVirtualFS().getRoot();
		StringBuilder sb = new StringBuilder();
		while (!root.equals(current) && current != null)
		{
			sb.append("../");
			current = current.getParent();
		}
		return sb.toString();
	}

	private static String encode(String s1)
	{
		try
		{
			return URLEncoder.encode(s1, "UTF-8");
		}
		catch (UnsupportedEncodingException e)
		{
			e.printStackTrace();
			return s1;
		}
	}

	public static String createRelativePathWeb(VirtualNode virtualNode, VirtualNode current)
	{
		if (virtualNode instanceof VirtualDirectory)
			return createRelativePathWeb((VirtualDirectory) virtualNode, current);
		else
			return createRelativePathWeb(((VirtualFile) virtualNode).getParent(), current);
	}

	public static String createRelativePathWeb(VirtualDirectory virtualDirectory,
			VirtualNode current)
	{
		if (virtualDirectory.equals(current))
			return "";
		List<VirtualNode> currentParents = new ArrayList<VirtualNode>();
		currentParents.add(current);
		while (current.getParent() != null)
		{
			currentParents.add(current.getParent());
			current = current.getParent();
		}
		List<VirtualNode> path = new ArrayList<VirtualNode>();
		int min = Integer.MAX_VALUE;
		while (virtualDirectory.getParent() != null)
		{
			int index = currentParents.indexOf(virtualDirectory);
			if (index >= 0 && index < min)
			{
				min = index;
				break;
			}
			path.add(0, virtualDirectory);
			virtualDirectory = virtualDirectory.getParent();
		}
		StringBuilder p = new StringBuilder();
		if (min != Integer.MAX_VALUE)
		{
			for (int i = 0; i < min; i++)
				p.append("../");
		}
		for (VirtualNode v : path)
			p.append(encode(v.getAttributes().getURLName())).append("/");
		return p.toString();
	}

	private static Link[] doNav(Site site, String webRoot, VirtualDirectory root,
			VirtualNode current, int depth, int from, File source) throws IOException
	{
		VirtualDirectory currentDir;
		if (current instanceof VirtualFile)
			currentDir = current.getParent();
		else
			currentDir = (VirtualDirectory) current;
		int level = Utils.countLevels(root, currentDir);
		if (level >= from)
			return doNav(site, webRoot, currentDir, depth, source);
		else
			return new Link[] {};
	}

	private static Link[] doNav(Site site, String webRoot, VirtualDirectory vdir, int depth,
			File source) throws IOException
	{
		if (depth == 0)
			return new Link[] {};
		List<Link> tmp = new ArrayList<Link>();
		for (VirtualDirectory vdir2 : vdir.getVisibleDirectories())
		{
			if (vdir2.isProtected())
				continue;
			Link[] childLinks = doNav(site, webRoot, vdir2, depth - 1, source);
			Link link = new Link(Utils.makeURL(webRoot, vdir2), vdir2.getAttributes()
					.getVisibleName(), childLinks);
			if (!vdir2.getAttributes().isDate())
			{
				site.getCache().addDependency(source, vdir2.getBackingFile());
				tmp.add(link);
			}
		}
		return tmp.toArray(new Link[] {});
	}

	private static Link[] doBreadcrumbs(Site site, String webRoot,
			VirtualNode contentDetailedNavDir, File source) throws IOException
	{
		ArrayList<Link> tmp = new ArrayList<Link>();
		if (contentDetailedNavDir == null)
			return new Link[] {};
		VirtualDirectory parent;
		if (contentDetailedNavDir instanceof VirtualFile)
			parent = ((VirtualFile) contentDetailedNavDir).getParent();
		else
			parent = (VirtualDirectory) contentDetailedNavDir;
		while (parent != null && !parent.isRoot())
		{
			tmp.add(0, new Link(Utils.makeURL(webRoot, parent), parent.getAttributes()
					.getVisibleName(), new Link[] {}));
			site.getCache().addDependency(source, parent.getBackingFile());
			parent = parent.getParent();
		}
		return tmp.toArray(new Link[] {});
	}

	public static Document createDocument(Site site, VirtualNode virtualDirectory,
			VirtualNode linkTo, VirtualNode current, int depth)
			throws UnsupportedEncodingException
	{
		String name=null;
		if(linkTo instanceof VirtualFile)
			name = FilenameUtils.getBaseName(linkTo.getAttributes().getVisibleName());
		else
		{
			if(!((VirtualDirectory)linkTo).isRoot())
				name = linkTo.getAttributes().getVisibleName();
		}
		int level=Utils.countLevels(current, linkTo);
		VirtualNode tmp=linkTo;
		for (int i = 1; i < level && name!=null; i++)
		{
			level=Utils.countLevels(current, linkTo);
			tmp=tmp.getParent();
			name += " / "
					+ FilenameUtils
							.getBaseName(tmp.getAttributes().getVisibleName());
		}
		System.err.println("NAAMEE: "+name+" current: "+virtualDirectory);
		SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy");
		String date = virtualDirectory.getAttributes().isDate() ? sdf.format(new Date(
				virtualDirectory.getAttributes().getSortNr())) : null;
		if (linkTo == null || linkTo.equals(current))
			return new Document(name, null, date, virtualDirectory.getAttributes().getSortNr());
		String link = createLink(site, linkTo, current);
		if(linkTo instanceof VirtualFile)
			link+=linkTo.getAttributes().getURLName()+".html";
		return new Document(name, link, date, virtualDirectory.getAttributes().getSortNr());
	}

	private static String createLink(Site site, VirtualNode linkTo, VirtualNode current)
			throws UnsupportedEncodingException
	{
		if (linkTo == null)
			return null;
		return createRelativePathWeb(linkTo, current);
	}

	public static byte[] makeHash(String s1)
	{
		try
		{
			MessageDigest md = MessageDigest.getInstance("SHA");
			md.update(s1.getBytes());
			return md.digest();
		}
		catch (NoSuchAlgorithmException e)
		{
			throw new RuntimeException("couldn't make digest of partial content");
		}
	}
}
