package net.xdocc;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

import net.xdocc.virtualfilesystem.VirtualDirectory;
import net.xdocc.virtualfilesystem.VirtualFile;
import net.xdocc.virtualfilesystem.VirtualNode;

public class ViewHandler implements ContentCompiler
{
	final private Site site;
	final private VirtualFile virtualNode;

	public ViewHandler(Site site, VirtualFile virtualNode)
	{
		this.site = site;
		this.virtualNode = virtualNode;
	}

	@Override
	public Boolean call() throws Exception
	{
		if (!virtualNode.getBackingFile().exists())
		{
			System.err.println("the file " + virtualNode + " does not exist!!");
			return false;
		}
		// do not compile files that do not have a date, these will
		// be compiled in index.dir.html
		if ((virtualNode.getParent().getVisibleFiles().size() <= 1 || !virtualNode.getParent()
				.hasFullEntries())
				&& (virtualNode.getParent().getAttributes().isVisible() || virtualNode.getParent()
						.isRoot()))
		{
			return false;
		}
		File generated = Utils.createGenerated(site, virtualNode, virtualNode.getAttributes()
				.getURLName()
				+ ".html");
		if (!site.getCache().isValid(virtualNode.getBackingFile(), generated))
		{
			List<Document> tmp = viewCompiled(site, virtualNode, virtualNode, virtualNode
					.getParent());
			Utils.writeHTML(site, virtualNode, virtualNode.getParent(), tmp
					.toArray(new Document[] {}), generated);
		}
		return true;
	}

	@Override
	public List<Document> viewCompiled(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current) throws Exception
	{
		PropertyReader pr = new PropertyReader(virtualNode.getBackingFile());
		int size;
		try
		{
			size = Integer.parseInt(pr.get("include_size"));
			size &= 0x7fffffff;
		}
		catch (NumberFormatException nfe)
		{
			size = Integer.MAX_VALUE;
		}
		List<Properties> list = pr.getTag("include_dir");
		List<Document> tmp = new ArrayList<Document>();
		for (Properties p : list)
		{
			String dir = PropertyReader.getKey(p);
			String mode = p.getProperty("mode");
			Collection<? extends VirtualNode> vdirs = virtualNode.find(dir);
			if (vdirs == null)
				continue;
			for (VirtualNode vNode : vdirs)
			{
				if (vNode instanceof VirtualDirectory)
				{
					DirectoryCompiler dc = new DirectoryCompiler(site, (VirtualDirectory) vNode,
							current);
					tmp.addAll(dc.view((VirtualDirectory) vNode, true, "preview".equals(mode)));
				}
			}
		}
		Collections.sort(tmp);
		return tmp.subList(0, Math.min(size, tmp.size()));
	}

	@Override
	public net.xdocc.Entry view(Site site, VirtualNode virtualNode, VirtualNode linkTo,
			VirtualDirectory current) throws Exception
	{
		return null;
	}
}
