package net.xdocc.virtualfilesystem;
import java.util.Set;

public class Attributes
{
	final private String visibleName;
	private String urlName;
	final private String name;
	final private Set<String> tags;
	final private long sortNr;
	final private boolean isDate;
	final private boolean isVisible;

	public Attributes(String visibleName, String name, Set<String> tags, long sortNr,
			boolean isDate, boolean isVisible)
	{
		this.visibleName = visibleName;
		this.name = name;
		this.tags = tags;
		this.sortNr = sortNr;
		this.isDate = isDate;
		this.isVisible = isVisible;
		this.setUrlName(createShort());
	}

	String createShort()
	{
		if (isVisible)
		{
			for (String tag : tags)
			{
				tag = tag.trim().toLowerCase();
				if (tag.startsWith("url="))
					return tag.substring(4);
			}
			StringBuilder sb = new StringBuilder();
			String[] tmp = visibleName.split("[^a-zA-Z0-9.]");
			for (int i = 0; i < tmp.length; i++)
			{
				if (i != 0)
					sb.append("_");
				sb.append(tmp[i].toLowerCase());
			}
			return sb.toString();
		}
		else
			return visibleName;
	}

	public String getVisibleName()
	{
		return visibleName;
	}

	public String getName()
	{
		return name;
	}

	public Set<String> getTags()
	{
		return tags;
	}

	public long getSortNr()
	{
		return sortNr;
	}

	public boolean isDate()
	{
		return isDate;
	}

	public boolean isVisible()
	{
		return isVisible;
	}

	@Override
	public String toString()
	{
		return "Visible name: " + visibleName + ", nr: " + sortNr;
	}

	public String getURLName()
	{
		return urlName;
	}

	public void setUrlName(String urlName)
	{
		this.urlName = urlName;
	}
}
