package net.xdocc.virtualfilesystem;
import java.io.File;
import java.util.Date;

class FileStat
{
	final private File file;
	final private boolean isFile;
	final private boolean isDirectory;
	private long lastModified;

	FileStat(File file)
	{
		this(file, false);
	}

	FileStat(File file, boolean forceRefresh)
	{
		this.file = file;
		this.isFile = file.isFile();
		this.isDirectory = file.isDirectory();
		if (!forceRefresh)
			updateLastModified();
	}

	void updateLastModified()
	{
		this.lastModified = file.lastModified();
	}

	long lastStoredModifiedTime()
	{
		return lastModified;
	}

	long lastNewModified()
	{
		return file.lastModified();
	}

	File getFile()
	{
		return file;
	}

	boolean isFile()
	{
		return isFile;
	}

	boolean isDirectory()
	{
		return isDirectory;
	}

	@Override
	public int hashCode()
	{
		return file.hashCode();
	}

	@Override
	public boolean equals(Object obj)
	{
		if (!(obj instanceof FileStat))
			return false;
		return ((FileStat) obj).file.equals(file);
	}

	@Override
	public String toString()
	{
		return "File :" + file.toString() + ", isFile=" + isFile + ", modified: "
				+ new Date(lastModified);
	}

	public boolean isForceRefresh()
	{
		// TODO Auto-generated method stub
		return false;
	}
}
