package net.xdocc.virtualfilesystem;
import java.io.File;
import java.io.IOException;

import net.xdocc.Utils;

public class VirtualFile extends VirtualNode
{
	final private String[] extensions;
	private boolean preview;

	public VirtualFile(File backingFile, VirtualDirectory parent, VirtualFS virtualFS)
			throws IOException
	{
		super(backingFile, parent, virtualFS);
		if (!backingFile.isFile())
			throw new IllegalArgumentException("File " + backingFile + " is not a file");
		this.extensions = Utils.getExtensions(getAttributes().getVisibleName());
		if (parent != null)
			parent.addFile(this);
	}

	public String[] getExtensions()
	{
		return extensions;
	}

	public void setPreview(boolean preview)
	{
		this.preview = preview;
	}

	public boolean isPreview()
	{
		return preview;
	}

	public String getRelativeVisiblePath()
	{
		StringBuilder sb = new StringBuilder();
		VirtualDirectory vdir = getParent();
		while (vdir.getParent() != null)
		{
			sb.insert(0, "/");
			sb.insert(0, vdir.getAttributes().getURLName());
			vdir = vdir.getParent();
		}
		return sb.toString();
	}
}
