package net.xdocc.virtualfilesystem;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class VirtualUtils
{
	public static Attributes parseFileName(String fileName)
	{
		
		boolean isDate = false;
		if(fileName.startsWith("03"))
			isDate=false;
		long sortNr=0;
		boolean isVisible = false;
		String tmpRest = fileName;
		Set<String> tags = new HashSet<String>();
		//
		Pattern p = Pattern.compile("^(([\\d]+[-/.]){1,2}[\\d]+)-");
		Matcher m = p.matcher(fileName);
		if (m.find())
		{
			String tmpDate = fileName.substring(m.start(1), m.end(1));
			tmpRest = fileName.substring(m.end()).trim();
			Date date = parseDate(tmpDate);
			isDate = true;
			isVisible = true;
			sortNr = date.getTime();
		}
		else
		{
			p = Pattern.compile("^([\\d]+)-");
			m = p.matcher(fileName);
			if (m.find())
			{
				String tmpNumber = fileName.substring(m.start(1), m.end(1));
				tmpRest = fileName.substring(m.end()).trim();
				sortNr = Long.parseLong(tmpNumber);
				isVisible = true;
			}
		}
		// tags
		p = Pattern.compile("\\[([^]\\[]*)]([.][a-z]+)*$");
		m = p.matcher(tmpRest);
		if (m.find())
		{
			String tmpTags = tmpRest.substring(m.start(1), m.end(1));
			tmpRest = tmpRest.substring(0, m.start(1) - 1).trim() + tmpRest.substring(m.end(1) + 1);
			parseTags(tmpTags, tags);
		}
		return new Attributes(tmpRest, fileName, tags, sortNr, isDate, isVisible);
	}
	
	static void parseTags(String tmpTags, Set<String> tags)
	{
		StringTokenizer st = new StringTokenizer(tmpTags, ",");
		while (st.hasMoreTokens())
		{
			String tag = st.nextToken();
			tags.add(tag.trim());
		}
	}

	static Date parseDate(String tmpDate)
	{
		Date date = parseDate(tmpDate, "y/M/d");
		if (date == null)
			date = parseDate(tmpDate, "y.M.d");
		if (date == null)
			date = parseDate(tmpDate, "y-M-d");
		if (date == null)
			date = parseDate(tmpDate, "y/M");
		if (date == null)
			date = parseDate(tmpDate, "y.M");
		if (date == null)
			date = parseDate(tmpDate, "y-M");
		return date;
	}

	static Date parseDate(String tmpDate, String format)
	{
		SimpleDateFormat df1 = new SimpleDateFormat(format);
		try
		{
			return df1.parse(tmpDate);
		}
		catch (ParseException e)
		{
			return null;
		}
	}
}
